<?php

namespace Drupal\paragraphs_collection\Plugin\paragraphs\Behavior;

use Drupal\Core\Asset\LibraryDiscoveryInterface;
use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Entity\EntityFieldManager;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Render\Element;
use Drupal\paragraphs\Entity\Paragraph;
use Drupal\paragraphs\ParagraphInterface;
use Drupal\paragraphs\ParagraphsBehaviorBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a way to hide specific paragraphs depending on the current language.
 *
 * @ParagraphsBehavior(
 *   id = "language",
 *   label = @Translation("Visibility per language"),
 *   description = @Translation("Restricts visibility of a paragraph per language. Usage on children of a container paragraph which uses a container behavior like Grid layout can have unexpected visual results."),
 *   weight = 0
 * )
 */
class ParagraphsLanguagePlugin extends ParagraphsBehaviorBase {

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The library discovery.
   *
   * @var \Drupal\Core\Asset\LibraryDiscoveryInterface $library_discovery
   */
  protected $libraryDiscovery;

  /**
   * ParagraphsLanguagePlugin constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityFieldManager $entity_field_manager
   *   The entity field manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Asset\LibraryDiscoveryInterface $library_discovery
   *   The library discovery.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityFieldManager $entity_field_manager, LanguageManagerInterface $language_manager, LibraryDiscoveryInterface $library_discovery) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_field_manager);

    $this->languageManager = $language_manager;
    $this->libraryDiscovery = $library_discovery;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition,
      $container->get('entity_field.manager'),
      $container->get('language_manager'),
      $container->get('library.discovery')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildBehaviorForm(ParagraphInterface $paragraph, array &$form, FormStateInterface $form_state) {
    if (!$this->languageManager->isMultilingual()) {
      return [];
    }

    foreach ($this->languageManager->getLanguages() as $language_code => $language) {
      $options[$language_code] = $language->getName();
    }

    $form['container'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['paragraphs-plugin-inline-container'],
      ],
    ];

    $form['container']['visibility'] = [
      '#type' => 'select',
      '#title' => $this->t('Language visibility'),
      '#options' => [
        'always' => $this->t('- Always visible -'),
        'hide' => $this->t('Hide for'),
        'show' => $this->t('Show for'),
      ],
      '#default_value' => $paragraph->getBehaviorSetting($this->getPluginId(), ['container', 'visibility']),
      '#multiple' => FALSE,
      '#attributes' => [
        'id' => ['paragraphs-behavior-language-behavior-form-visibility-' . $paragraph->id()],
        'class' => ['paragraphs-plugin-form-element']
      ],
    ];

    $form['container']['languages'] = [
      '#type' => 'select',
      '#options' => $options,
      '#empty_option' => $this->t('- None -'),
      '#empty_value' => 'none',
      '#default_value' => $paragraph->getBehaviorSetting($this->getPluginId(), ['container', 'languages']),
      '#states' => [
        'invisible' => [
          ':input[id="paragraphs-behavior-language-behavior-form-visibility-' . $paragraph->id() . '"]' => ['value' => 'always'],
        ],
      ],
      '#multiple' => TRUE,
      '#attributes' => [
        'class' => ['paragraphs-behavior-language-behavior-form-languages', 'paragraphs-plugin-form-element'],
      ],
    ];

    if ($this->libraryDiscovery->getLibraryByName('paragraphs_collection', 'language')) {
      $form['container']['languages']['#attached']['library'] = [
        'paragraphs_collection/language',
      ];
    }

    $form['#attached']['library'][] = 'paragraphs_collection/plugin_admin';
    $form['container']['#attributes']['class'][] = 'paragraphs-behavior-language-behavior-form';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function view(array &$build, Paragraph $paragraphs_entity, EntityViewDisplayInterface $display, $view_mode) {
    $visibility = $paragraphs_entity->getBehaviorSetting($this->getPluginId(), ['container', 'visibility']);
    $languages = $paragraphs_entity->getBehaviorSetting($this->getPluginId(), ['container', 'languages']) ?: [];
    $current_language = $this->languageManager->getCurrentLanguage();

    // In the 'always' visibility mode: Do not hide the paragraph.
    if (!$visibility || $visibility == 'always') {
      return $build;
    }

    $show = TRUE;

    // In the 'show' visibility mode: Hide the paragraph, if the current
    // language is not among the selected ones.
    if ($visibility == 'show' && !in_array($current_language->getId(), $languages)) {
      $show = FALSE;
    }

    // In the 'hide' visibility mode: Hide the paragraph, if the current
    // language is among the selected ones.
    if ($visibility == 'hide' && in_array($current_language->getId(), $languages)) {
      $show = FALSE;
    }

    if (!$show) {
      foreach (Element::children($build) as $children) {
        $build[$children]['#access'] = FALSE;
      }
    }

    return $build;
  }

}
